@@ -66,11 +66,16 @@ class ImmunizationStatusReport(Report):
                 dose_number, dose_age, age_unit = dose.dose_number, \
                     dose.age_dose, dose.age_unit
                 
-                p_age = patient.patient_age(name='raw_age')
-                
+                #TODO : For 3.2, use the generic raw_age argument
+                # from compute_age_from_dates
+
+                p_age = [patient.age.split(' ')[0][:-1], 
+                    patient.age.split(' ')[1][:-1], 
+                    patient.age.split(' ')[2][:-1]]                
+
                 #Age of the person in years and months
-                pyears,pmonths = p_age[0],p_age[1]
-                
+                pyears,pmonths = int(p_age[0]),int(p_age[1])
+
                 pmonths = (pyears*12)+pmonths
                 
                 if ((age_unit == 'months' and pmonths >= dose_age) or
