@@ -2,8 +2,8 @@
 ##############################################################################
 #
 #    GNU Health: The Free Health and Hospital Information System
-#    Copyright (C) 2008-2015 Luis Falcon <lfalcon@gnusolidario.org>
-#    Copyright (C) 2011-2015 GNU Solidario <health@gnusolidario.org>
+#    Copyright (C) 2008-2016 Luis Falcon <lfalcon@gnusolidario.org>
+#    Copyright (C) 2011-2016 GNU Solidario <health@gnusolidario.org>
 #
 #
 #    This program is free software: you can redistribute it and/or modify
@@ -30,19 +30,24 @@ def register():
     Pool.register(
         InpatientSequences,
         DietTherapeutic,
-        DietBelief,
         InpatientRegistration,
         BedTransfer,
         Appointment,
+        PatientEvaluation,
+        ECG,
         PatientData,
         InpatientMedication,
         InpatientMedicationAdminTimes,
         InpatientMedicationLog,
         InpatientDiet,
         CreateBedTransferInit,
+        InpatientMeal,
+        InpatientMealOrder,
+        InpatientMealOrderItem,
         module='health_inpatient', type_='model')
     
     Pool.register(
         CreateBedTransfer,
+        CreateInpatientEvaluation,
         module='health_inpatient', type_='wizard')
 
