@@ -2,8 +2,8 @@
 ##############################################################################
 #
 #    GNU Health: The Free Health and Hospital Information System
-#    Copyright (C) 2008-2015 Luis Falcon <lfalcon@gnusolidario.org>
-#    Copyright (C) 2011-2015 GNU Solidario <health@gnusolidario.org>
+#    Copyright (C) 2008-2016 Luis Falcon <lfalcon@gnusolidario.org>
+#    Copyright (C) 2011-2016 GNU Solidario <health@gnusolidario.org>
 #
 #
 #    This program is free software: you can redistribute it and/or modify
@@ -21,16 +21,20 @@
 #
 ##############################################################################
 from datetime import datetime
-from trytond.model import ModelView, ModelSingleton, ModelSQL, fields
+from trytond.model import ModelView, ModelSingleton, ModelSQL, fields, Unique
 from trytond.transaction import Transaction
+from trytond.tools import grouped_slice, reduce_ids
+from sql import Literal, Table
 from trytond.pool import Pool
-from trytond.pyson import Eval, Not, Bool, And, Equal
+from trytond.pyson import Eval, Not, Bool, And, Equal, Or
+from trytond import backend
 
 
-__all__ = ['InpatientSequences', 'DietTherapeutic', 'DietBelief',
-    'InpatientRegistration', 'BedTransfer', 'Appointment', 'PatientData',
+__all__ = ['InpatientSequences', 'DietTherapeutic','InpatientRegistration', 
+    'BedTransfer', 'Appointment', 'PatientEvaluation', 'PatientData',
     'InpatientMedication', 'InpatientMedicationAdminTimes',
-    'InpatientMedicationLog', 'InpatientDiet']
+    'InpatientMedicationLog', 'InpatientDiet', 'InpatientMeal',
+    'InpatientMealOrder','InpatientMealOrderItem', 'ECG']
 
 
 class InpatientSequences(ModelSingleton, ModelSQL, ModelView):
@@ -41,6 +45,10 @@ class InpatientSequences(ModelSingleton, ModelSQL, ModelView):
         'ir.sequence', 'Inpatient Sequence', required=True,
         domain=[('code', '=', 'gnuhealth.inpatient.registration')]))
 
+    inpatient_meal_order_sequence = fields.Property(fields.Many2One(
+        'ir.sequence', 'Inpatient Meal Sequence', required=True,
+        domain=[('code', '=', 'gnuhealth.inpatient.meal.order')]))
+
 
 # Therapeutic Diet types
 
@@ -56,96 +64,99 @@ class DietTherapeutic (ModelSQL, ModelView):
     def __setup__(cls):
         super(DietTherapeutic, cls).__setup__()
 
+        t = cls.__table__()
         cls._sql_constraints = [
-            ('code_uniq', 'unique(code)',
-             'The Diet code already exists')]
-
-
-# Diet by belief / religion
-
-class DietBelief (ModelSQL, ModelView):
-    'Diet by Belief'
-    __name__="gnuhealth.diet.belief"
-
-    name = fields.Char('Belief', required=True, translate=True)
-    code = fields.Char('Code', required=True)
-    description = fields.Text('Description', required=True, translate=True)
-
-    @classmethod
-    def __setup__(cls):
-        super(DietBelief, cls).__setup__()
-
-        cls._sql_constraints = [
-            ('code_uniq', 'unique(code)',
-             'The Diet code already exists')]
+            ('code_unique', Unique(t,t.code),
+                'The Diet code already exists'),
+            ]
 
 
 class InpatientRegistration(ModelSQL, ModelView):
     'Patient admission History'
     __name__ = 'gnuhealth.inpatient.registration'
 
+    STATES = {'readonly': Or(Eval('state') == 'done',
+        Eval('state') == 'finished')}
+    
     name = fields.Char('Registration Code', readonly=True, select=True)
     patient = fields.Many2One('gnuhealth.patient', 'Patient',
-     required=True, select=True)
+     required=True, select=True, states = STATES)
     admission_type = fields.Selection([
+        (None, ''),
         ('routine', 'Routine'),
         ('maternity', 'Maternity'),
         ('elective', 'Elective'),
         ('urgent', 'Urgent'),
         ('emergency', 'Emergency'),
-        ], 'Admission type', required=True, select=True)
+        ], 'Admission type', required=True, select=True, states = STATES)
     hospitalization_date = fields.DateTime('Hospitalization date',
-        required=True, select=True)
+        required=True, select=True, states = STATES)
     discharge_date = fields.DateTime('Expected Discharge Date', required=True,
-        select=True)
+        states = STATES)
     attending_physician = fields.Many2One('gnuhealth.healthprofessional',
-        'Attending Physician', select=True)
+        'Attending Physician',  states = STATES)
     operating_physician = fields.Many2One('gnuhealth.healthprofessional',
-        'Operating Physician')
+        'Operating Physician',  states = STATES)
     admission_reason = fields.Many2One('gnuhealth.pathology',
-        'Reason for Admission', help="Reason for Admission", select=True)
+        'Reason for Admission', help="Reason for Admission",  states = STATES,
+        select=True)
     bed = fields.Many2One('gnuhealth.hospital.bed', 'Hospital Bed',
         states={
             'required': Not(Bool(Eval('name'))),
-            'readonly': Bool(Eval('name')),
+            'readonly': Or(Eval('state') == 'done',
+                            Eval('state') == 'finished', 
+                            Bool(Eval('name')),
+                        )
             },
         depends=['name'])
-    nursing_plan = fields.Text('Nursing Plan')
+    nursing_plan = fields.Text('Nursing Plan', states = STATES)
     medications = fields.One2Many('gnuhealth.inpatient.medication', 'name',
-        'Medications')
+        'Medications', states = STATES)
     therapeutic_diets = fields.One2Many('gnuhealth.inpatient.diet', 'name',
-        'Meals / Diet Program')
-    diet_belief = fields.Many2One('gnuhealth.diet.belief',
-        'Belief', help="Enter the patient belief or religion to choose the \
-            proper diet")
-    diet_vegetarian = fields.Selection((
-        ('none', 'None'),
-        ('vegetarian', 'Vegetarian'),
-        ('lacto', 'Lacto vegetarian'),
-        ('lactoovo', 'Lacto-ovo vegetarian'),
-        ('pescetarian', 'Pescetarian'),
-        ('vegan', 'Vegan'),
-        ), 'Vegetarian', sort=False, required=True)
-    nutrition_notes = fields.Text('Nutrition notes / directions')
-    discharge_plan = fields.Text('Discharge Plan')
-    info = fields.Text('Extra Info')
+        'Meals / Diet Program', states = STATES)
+        
+    nutrition_notes = fields.Text('Nutrition notes / directions', states = STATES)
+    discharge_plan = fields.Text('Discharge Plan', states = STATES)
+    info = fields.Text('Notes',states = STATES)
     state = fields.Selection((
+        (None, ''),
         ('free', 'free'),
         ('cancelled', 'cancelled'),
         ('confirmed', 'confirmed'),
         ('hospitalized', 'hospitalized'),
-        ('done','Done'),
-        ), 'Status', select=True)
+        ('done','Discharged - needs cleaning'),
+        ('finished','Finished'),
+        ), 'Status', select=True, readonly=True)
         
     bed_transfers = fields.One2Many('gnuhealth.bed.transfer', 'name',
         'Transfer History', readonly=True)
 
     discharged_by = fields.Many2One(
         'gnuhealth.healthprofessional', 'Discharged by', readonly=True,
-        states={'invisible': Not(Equal(Eval('state'), 'done'))},
         help="Health Professional that discharged the patient")
 
-    institution = fields.Many2One('gnuhealth.institution', 'Institution')
+    discharge_reason = fields.Selection([
+        (None, ''),
+        ('home','Home / Selfcare'),
+        ('transfer','Transferred to another institution'),
+        ('death','Death'),
+        ('against_advice','Left against medical advice')],
+        'Discharge Reason', 
+        states={'readonly': Not(Equal(Eval('state'), 'hospitalized')),},
+        help="Reason for patient discharge")
+
+    discharge_dx = fields.Many2One('gnuhealth.pathology',
+        'Discharge Dx', help="Code for Discharge Diagnosis",
+        states={'readonly': Not(Equal(Eval('state'), 'hospitalized')),})
+  
+    institution = fields.Many2One('gnuhealth.institution', 'Institution',
+        readonly=True)
+
+    puid = fields.Function(
+        fields.Char('PUID', help="Person Unique Identifier"),
+        'get_patient_puid', searcher="search_patient_puid")
+
+
 
     @staticmethod
     def default_institution():
@@ -153,15 +164,48 @@ class InpatientRegistration(ModelSQL, ModelView):
         institution = HealthInst.get_institution()
         return institution
 
+    def get_patient_puid(self, name):
+        return self.patient.name.ref
+
+    @classmethod
+    def search_patient_puid(cls, name, clause):
+        res = []
+        value = clause[2]
+        res.append(('patient.name.ref', clause[1], value))
+        return res
+
+
+    @classmethod
+    def __register__(cls, module_name):
+        super(InpatientRegistration, cls).__register__(module_name)
+
+        cursor = Transaction().cursor
+        TableHandler = backend.get('TableHandler')
+        table = TableHandler(cursor, cls, module_name)
+
+        # Update to version 3.0
+        # The diets based on religion and philosophy
+        # are now in the lifestyle module
+        if table.column_exist('diet_vegetarian'):
+            table.drop_column('diet_vegetarian')
+
+        if table.column_exist('diet_belief'):
+            table.drop_column('diet_belief')
+
+
     @classmethod
     def __setup__(cls):
         super(InpatientRegistration, cls).__setup__()
+        t = cls.__table__()
         cls._sql_constraints = [
-            ('name_uniq', 'unique(name)',
-             'The Registration code already exists')
-        ]
+            ('name_unique', Unique(t,t.name),
+                'The Registration code already exists'),
+            ]
+
         cls._error_messages.update({
                 'bed_is_not_available': 'Bed is not available',
+                'discharge_reason_needed': 'Admission and Discharge reasons \n'
+                'as well as Discharge Dx are needed',
                 'destination_bed_unavailable': 'Destination bed unavailable'})
         cls._buttons.update({
                 'confirmed': {
@@ -171,16 +215,20 @@ class InpatientRegistration(ModelSQL, ModelView):
                 'cancel': {
                     'invisible': Not(Equal(Eval('state'), 'confirmed')),
                     },
+                'admission': {
+                    'invisible': Not(Equal(Eval('state'), 'confirmed')),
+                    },
                 'discharge': {
                     'invisible': Not(Equal(Eval('state'), 'hospitalized')),
                     },
-                'admission': {
-                    'invisible': Not(Equal(Eval('state'), 'confirmed')),
+                'bedclean': {
+                    'invisible': Not(Equal(Eval('state'), 'done')),
                     },
                 })
 
     ## Method to check for availability and make the hospital bed reservation
-
+    # Checks that there are not overlapping dates and status of the bed / room
+    # is not confirmed, hospitalized or done but requiring cleaning ('done')
     @classmethod
     @ModelView.button
     def confirmed(cls, registrations):
@@ -192,11 +240,11 @@ class InpatientRegistration(ModelSQL, ModelView):
             FROM gnuhealth_inpatient_registration \
             WHERE (hospitalization_date::timestamp,discharge_date::timestamp) \
                 OVERLAPS (timestamp %s, timestamp %s) \
-              AND (state = %s or state = %s) \
+              AND (state = %s or state = %s or state = %s) \
               AND bed = CAST(%s AS INTEGER) ",
             (registration_id.hospitalization_date,
             registration_id.discharge_date,
-            'confirmed', 'hospitalized', str(bed_id)))
+            'confirmed', 'hospitalized', 'done', str(bed_id)))
         res = cursor.fetchone()
         if (registration_id.discharge_date.date() <
             registration_id.hospitalization_date.date()):
@@ -222,6 +270,16 @@ class InpatientRegistration(ModelSQL, ModelView):
         cls.write(registrations, {'state': 'done',
             'discharged_by': signing_hp})
 
+        Bed.write([registration_id.bed], {'state': 'to_clean'})
+
+    @classmethod
+    @ModelView.button
+    def bedclean(cls, registrations):
+        registration_id = registrations[0]
+        Bed = Pool().get('gnuhealth.hospital.bed')
+
+        cls.write(registrations, {'state': 'finished'})
+
         Bed.write([registration_id.bed], {'state': 'free'})
 
     @classmethod
@@ -259,33 +317,41 @@ class InpatientRegistration(ModelSQL, ModelView):
                     config.inpatient_registration_sequence.id)
         return super(InpatientRegistration, cls).create(vlist)
 
-    @classmethod
-    def write(cls, registrations, vals):
-        # Don't allow to write the record if the evaluation has been done
-        if registrations[0].state == 'done':
-            cls.raise_user_error(
-                "This hospitalization is at state Done\n"
-                "You can no longer modify it.")
-        return super(InpatientRegistration, cls).write(registrations, vals)
-
     @staticmethod
     def default_state():
         return 'free'
 
-    # Allow searching by the hospitalization code or patient name
+    @classmethod
+    def validate(cls, registrations):
+        super(InpatientRegistration, cls).validate(registrations)
+        for registration in registrations:
+            registration.check_discharge_context()
 
+
+    def check_discharge_context(self):
+        if ((not self.discharge_reason or not self.discharge_dx
+            or not self.admission_reason)
+            and self.state == 'done'):
+                self.raise_user_error('discharge_reason_needed')
+
+    
+    # Format Registration ID : Patient : Bed
     def get_rec_name(self, name):
         if self.patient:
-            return self.name + ': ' + self.patient.name.name + ' ' + \
-             self.patient.name.lastname
+            return self.name + ':' + self.bed.rec_name + ':' \
+                + self.patient.rec_name 
+                
         else:
             return self.name
 
+    # Allow searching by the hospitalization code, patient name
+    # or bed number 
+
     @classmethod
     def search_rec_name(cls, name, clause):
         field = None
         # Search by Registration Code ID or Patient
-        for field in ('name', 'patient'):
+        for field in ('name', 'patient', 'bed'):
             registrations = cls.search([(field,) + tuple(clause[1:])], limit=1)
             if registrations:
                 break
@@ -294,6 +360,8 @@ class InpatientRegistration(ModelSQL, ModelView):
         return [(cls._rec_name,) + tuple(clause[1:])]
 
 
+
+        
 class BedTransfer(ModelSQL, ModelView):
     'Bed transfers'
     __name__ = 'gnuhealth.bed.transfer'
@@ -309,13 +377,89 @@ class BedTransfer(ModelSQL, ModelView):
 
 
 class Appointment(ModelSQL, ModelView):
-    'Add Inpatient Registration field to the Appointment model.'
     __name__ = 'gnuhealth.appointment'
 
     inpatient_registration_code = fields.Many2One(
         'gnuhealth.inpatient.registration', 'Inpatient Registration',
         help="Enter the patient hospitalization code")
 
+class PatientEvaluation(ModelSQL, ModelView):
+    __name__ = 'gnuhealth.patient.evaluation'
+
+    inpatient_registration_code = fields.Many2One(
+        'gnuhealth.inpatient.registration', 'IPC',
+        help="Enter the patient hospitalization code")
+
+class ECG(ModelSQL, ModelView):
+    __name__ = 'gnuhealth.patient.ecg'
+
+    inpatient_registration_code = fields.Many2One(
+        'gnuhealth.inpatient.registration', 'Inpatient Registration',
+        help="Enter the patient hospitalization code")
+
+    @classmethod
+    def __register__(cls, module_name):
+
+        # Upgrade to release 3.0 
+        # Move existing ICU EGC records to the generic gnuhealth.patient.ecg
+        # model
+        
+        super(ECG, cls).__register__(module_name)
+
+        cursor = Transaction().cursor
+        TableHandler = backend.get('TableHandler')
+
+        
+        if TableHandler.table_exist(cursor, 'gnuhealth_icu_ecg'):
+
+            table = TableHandler(cursor, cls, module_name)
+
+            # Retrieve IDs from ECGs at ICU
+
+            cursor.execute('select id,name from gnuhealth_icu_ecg')
+            
+            icu_ecg_ids = cursor.fetchall()
+
+            # Traverse each record on the icu_ecg table
+            for icu_ecg in icu_ecg_ids:
+
+                registration = str(icu_ecg[1])
+
+                # Get the patient ID related to the registration
+                cursor.execute('select patient FROM \
+                gnuhealth_inpatient_registration \
+                where id = %s',registration)
+                
+                patient_id = cursor.fetchone()
+
+                cursor.execute('select name, ecg_date, lead, axis, rate,\
+                rhythm, pacemaker, pr, qrs, qt, st_segment, twave_inversion, \
+                interpretation, ecg_strip FROM gnuhealth_icu_ecg where \
+                name = %s and id = %s', (registration, icu_ecg[0]))
+                
+                ecg = cursor.fetchone()
+                                
+                (name, ecg_date, lead, axis, rate, rhythm, \
+                pacemaker, pr, qrs, qt, st_segment, twave_inversion, \
+                interpretation, ecg_strip) = ecg
+
+                # Insert the values on the new ECG table
+                cursor.execute('insert into gnuhealth_patient_ecg \
+                (name, inpatient_registration_code, ecg_date, lead, axis, \
+                rate,rhythm,pacemaker, pr, qrs, qt, st_segment, \
+                twave_inversion, interpretation, ecg_strip) \
+                VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)',
+                (patient_id, registration, ecg_date, lead, axis, rate, \
+                rhythm, pacemaker, pr, qrs, qt, st_segment, \
+                twave_inversion, interpretation, ecg_strip))
+
+
+                # Rename obsolete table gnuhealth_icu_ecg
+                # Once we know everything is OK we can drop it
+                
+                cursor.execute('alter table gnuhealth_icu_ecg rename \
+                to old_icu_ecg')
+                
 
 class PatientData(ModelSQL, ModelView):
     'Inherit patient model and add the patient status to the patient.'
@@ -323,35 +467,58 @@ class PatientData(ModelSQL, ModelView):
 
     patient_status = fields.Function(fields.Boolean('Hospitalized',
         help="Show the hospitalization status of the patient"),
-        'get_patient_status')
+        getter='get_patient_status', searcher='search_patient_status')
 
-    def get_patient_status(self, name):
-        
+    @classmethod
+    def get_patient_status(cls, patients, name):
         cursor = Transaction().cursor
+        pool = Pool()
+        Registration = pool.get('gnuhealth.inpatient.registration')
+        registration = Registration.__table__()
 
-        def get_hospitalization_status(patient_dbid):
-            is_hospitalized = False
+        # Will store statuses {patient: True/False, ...}
+        ids = map(int, patients)
+        result = dict.fromkeys(ids, False)
 
-            cursor.execute('SELECT count(state) \
-                FROM gnuhealth_inpatient_registration \
-                WHERE patient = %s \
-                AND state = \'hospitalized\' LIMIT 1' , (patient_dbid,))
-            
-            if cursor.fetchone()[0] > 0:
-                is_hospitalized = True
-            
-            return is_hospitalized
+        for sub_ids in grouped_slice(ids):
+            # SQL expression for relevant patient ids
+            clause_ids = reduce_ids(registration.id, sub_ids)
 
-        result = ''
+            # Hospitalized patient ids
+            query = registration.select(registration.patient, Literal(True),
+                where=(registration.state == 'hospitalized') & clause_ids,
+                group_by=registration.patient)
+
+            # Update dictionary of patient ids with True statuses
+            cursor.execute(*query)
+            result.update(cursor.fetchall())
 
-        # Get the patient (DB) id to be used in the search on the medical
-        # inpatient registration table lookup
-        
-        patient_dbid = self.id
-        if patient_dbid:
-            result = get_hospitalization_status(patient_dbid)
         return result
 
+    @classmethod
+    def search_patient_status(cls, name, clause):
+        p = Pool().get('gnuhealth.inpatient.registration')
+        table = p.__table__()
+        pat = cls.__table__()
+        _, operator, value = clause
+
+        # Validate operator and value
+        if operator not in ['=', '!=']:
+            raise ValueError('Wrong operator: %s' % operator)
+        if value is not True and value is not False:
+            raise ValueError('Wrong value: %s' % value)
+
+        # Find hospitalized patient ids
+        j = pat.join(table, condition = pat.id == table.patient)
+        s = j.select(pat.id, where = table.state == 'hospitalized')
+
+        # Choose domain operator
+        if (operator == '=' and value) or (operator == '!=' and not value):
+            d = 'in'
+        else:
+            d = 'not in'
+
+        return [('id', d, s)]
 
 class InpatientMedication (ModelSQL, ModelView):
     'Inpatient Medication'
@@ -497,3 +664,199 @@ class InpatientDiet (ModelSQL, ModelView):
     diet = fields.Many2One('gnuhealth.diet.therapeutic', 'Diet', required=True)
     remarks = fields.Text('Remarks / Directions',
         help='specific remarks for this diet / patient')
+
+class InpatientMeal (ModelSQL, ModelView):
+    'Inpatient Meal'
+    __name__="gnuhealth.inpatient.meal"
+
+    name = fields.Many2One(
+        'product.product', 'Food', required=True,
+        help='Food')
+
+    diet_therapeutic = fields.Many2One('gnuhealth.diet.therapeutic',
+        'Diet')
+
+    diet_belief = fields.Many2One('gnuhealth.diet.belief',
+        'Belief')
+
+    diet_vegetarian = fields.Many2One('gnuhealth.vegetarian_types',
+        'Vegetarian')
+
+    institution = fields.Many2One('gnuhealth.institution', 'Institution')
+
+    @staticmethod
+    def default_institution():
+        HealthInst = Pool().get('gnuhealth.institution')
+        institution = HealthInst.get_institution()
+        return institution
+
+    def get_rec_name(self, name):
+        if self.name:
+            return self.name.name
+
+class InpatientMealOrderItem (ModelSQL, ModelView):
+    'Inpatient Meal Item'
+    __name__="gnuhealth.inpatient.meal.order.item"
+
+    name = fields.Many2One('gnuhealth.inpatient.meal.order',
+        'Meal Order')
+
+    meal = fields.Many2One('gnuhealth.inpatient.meal', 'Meal')
+
+    remarks = fields.Char('Remarks')
+
+
+class InpatientMealOrder (ModelSQL, ModelView):
+    'Inpatient Meal Order'
+    __name__="gnuhealth.inpatient.meal.order"
+
+    name = fields.Many2One('gnuhealth.inpatient.registration',
+        'Registration Code', domain=[('state', '=', 'hospitalized')],
+        required=True)
+
+    mealtime = fields.Selection((
+        (None, ''),
+        ('breakfast', 'Breakfast'),
+        ('lunch', 'Lunch'),
+        ('dinner', 'Dinner'),
+        ('snack', 'Snack'),
+        ('special', 'Special order'),
+        ), 'Meal time', required=True, sort=False)
+
+    meal_item = fields.One2Many('gnuhealth.inpatient.meal.order.item', 'name',
+        'Items')
+
+    meal_order = fields.Char('Order', readonly=True)
+
+    health_professional = fields.Many2One('gnuhealth.healthprofessional',
+        'Health Professional')
+
+    remarks = fields.Text('Remarks')
+
+    meal_warning = fields.Boolean('Warning',readonly=True,
+        help="The patient has special needs on meals")
+
+    meal_warning_ack = fields.Boolean('Ack',
+        help="Check if you have verified the warnings on the"
+        " patient meal items")
+
+    order_date = fields.DateTime('Date',
+        help='Order date', required=True)
+
+    state = fields.Selection((
+        (None, ''),
+        ('draft', 'Draft'),
+        ('cancelled', 'Cancelled'),
+        ('ordered', 'Ordered'),
+        ('processing', 'Processing'),
+        ('done','Done'),
+        ), 'Status', readonly=True)
+
+    @staticmethod
+    def default_order_date():
+        return datetime.now()
+
+    @staticmethod
+    def default_state():
+        return 'draft'
+
+    @staticmethod
+    def default_health_professional():
+        pool = Pool()
+        return pool.get('gnuhealth.healthprofessional').get_health_professional()
+
+    @classmethod
+    def create(cls, vlist):
+        Sequence = Pool().get('ir.sequence')
+        Config = Pool().get('gnuhealth.sequences')
+
+        vlist = [x.copy() for x in vlist]
+        for values in vlist:
+            if not values.get('meal_order'):
+                config = Config(1)
+                values['meal_order'] = Sequence.get_id(
+                    config.inpatient_meal_order_sequence.id)
+        return super(InpatientMealOrder, cls).create(vlist)
+
+    @classmethod
+    def validate(cls, meal_orders):
+        super(InpatientMealOrder, cls).validate(meal_orders)
+        for meal_order in meal_orders:
+            meal_order.check_meal_order_warning()
+            meal_order.check_health_professional()
+            
+    def check_meal_order_warning(self):
+        if not self.meal_warning_ack:
+            self.raise_user_error('meal_order_warning')
+
+    def check_health_professional(self):
+        if not self.health_professional:
+            self.raise_user_error('health_professional_warning')
+
+
+    @fields.depends('name')
+    def on_change_name(self):
+        meal_warning = False
+        meal_warning_ack = True
+        if self.name:
+            # Trigger the warning if the patient 
+            # has special needs on meals (religion / philosophy )
+            if (self.name.patient.vegetarian_type or 
+                self.name.patient.diet_belief):
+                meal_warning = True
+                meal_warning_ack = False
+        return {
+            'meal_warning': meal_warning,
+            'meal_warning_ack': meal_warning_ack,
+        }
+
+    @classmethod
+    def __setup__(cls):
+        super(InpatientMealOrder, cls).__setup__()
+        cls._buttons.update({
+            'cancel': {'invisible': Not(Equal(Eval('state'), 'ordered'))}
+            })
+
+        cls._buttons.update({
+            'generate': {'invisible': Or(Equal(Eval('state'), 'ordered'),
+                    Equal(Eval('state'), 'done'))},
+            })
+
+        cls._buttons.update({
+            'done': {'invisible': Not(Equal(Eval('state'), 'ordered'))}
+            })
+
+        cls._error_messages.update({
+            'meal_order_warning':
+            '===== MEAL WARNING ! =====\n\n\n'
+            'This patient has special meal needs \n\n'
+            'Check and acknowledge that\n'
+            'the meal items in this order are correct \n\n',
+            'health_professional_warning':
+            'No health professional associated to this user',
+            })
+
+        t = cls.__table__()
+        cls._sql_constraints = [
+            ('meal_order_uniq', Unique(t,t.meal_order),
+                'The Meal Order code already exists'),
+            ]
+
+
+    @classmethod
+    @ModelView.button
+    def generate(cls, mealorders):
+        cls.write(mealorders, {
+            'state': 'ordered'})
+
+    @classmethod
+    @ModelView.button
+    def cancel(cls, mealorders):
+        cls.write(mealorders, {
+            'state': 'cancelled'})
+
+    @classmethod
+    @ModelView.button
+    def done(cls, mealorders):
+        cls.write(mealorders, {
+            'state': 'done'})
