@@ -2,8 +2,8 @@
 ##############################################################################
 #
 #    GNU Health: The Free Health and Hospital Information System
-#    Copyright (C) 2008-2015 Luis Falcon <lfalcon@gnusolidario.org>
-#    Copyright (C) 2011-2015 GNU Solidario <health@gnusolidario.org>
+#    Copyright (C) 2008-2016 Luis Falcon <lfalcon@gnusolidario.org>
+#    Copyright (C) 2011-2016 GNU Solidario <health@gnusolidario.org>
 #
 #
 #
@@ -38,6 +38,19 @@ class CreateBedTransferInit(ModelView):
         required=True, select=True)
     reason = fields.Char('Reason', required=True)
 
+    orig_bed_state = fields.Selection((
+        (None,''),
+        ('free', 'Free'),
+        ('reserved', 'Reserved'),
+        ('occupied', 'Occupied'),
+        ('to_clean', 'Needs cleaning'),
+        ('na', 'Not available'),
+        ), 'Bed of origin Status', sort=False, required=True)
+
+    @staticmethod
+    def default_orig_bed_state():
+        return 'to_clean'
+
 class CreateBedTransfer(Wizard):
     'Create Bed Transfer'
     __name__ = 'gnuhealth.bed.transfer.create'
@@ -74,11 +87,15 @@ class CreateBedTransfer(Wizard):
         current_bed = registration.bed
         destination_bed = self.start.newbed
         reason = self.start.reason
+        orig_bed_state = self.start.orig_bed_state
+
 
         # Check that the new bed is free
         if (destination_bed.state == 'free'):
-            # Free the current bed
-            bed.write([current_bed], {'state': 'free'})
+
+            # Update bed status with the one given in the transfer
+            bed.write([current_bed], {'state': orig_bed_state})
+
             # Set as occupied the new bed
             bed.write([destination_bed], {'state': 'occupied'})
             # Update the hospitalization record
